@@ -3,11 +3,15 @@
 #include "interactor_impl.h"
 #include "log.h"
 #include "options.h"
-#include "window.h"
+#include "window_impl.h"
+
+#include "vtkF3DRenderer.h"
 
 #include <vtkDoubleArray.h>
 #include <vtkImporter.h>
 #include <vtkProgressBarRepresentation.h>
+#include <vtkRenderWindow.h>
+#include <vtkRendererCollection.h>
 #include <vtkVersion.h>
 
 #include <cmath>
@@ -16,26 +20,48 @@
 namespace f3d::detail
 {
 //----------------------------------------------------------------------------
-bool animationManager::Initialize(
-  const options* options, window* window, interactor_impl* interactor, vtkImporter* importer)
+animationManager::animationManager(const options& options, window_impl& window)
+  : Options(options)
+  , Window(window)
 {
-  assert(importer);
+  if (vtksys::SystemTools::HasEnv("CTEST_F3D_PREVENT_SKIP_FRAME"))
+  {
+    this->SkipFrame = false;
+  }
+}
+
+//----------------------------------------------------------------------------
+void animationManager::SetImporter(vtkImporter* importer)
+{
+  this->Importer = importer;
+}
+
+//----------------------------------------------------------------------------
+void animationManager::SetInteractor(interactor_impl* interactor)
+{
+  this->Interactor = interactor;
+}
+
+//----------------------------------------------------------------------------
+void animationManager::SetDeltaTime(double deltaTime)
+{
+  this->DeltaTime = deltaTime;
+}
+
+//----------------------------------------------------------------------------
+void animationManager::Initialize()
+{
+  assert(this->Importer);
   this->HasAnimation = false;
   this->Playing = false;
   this->CurrentTime = 0;
   this->CurrentTimeSet = false;
-  this->Options = options;
-  this->Interactor = interactor;
-  this->Window = window;
-  this->Importer = importer;
 
-  // This can be -1 if animation support is not implemented in the importer
   this->AvailAnimations = this->Importer->GetNumberOfAnimations();
-
-  if (this->AvailAnimations > 0 && interactor)
+  if (this->AvailAnimations > 0 && this->Interactor)
   {
     this->ProgressWidget = vtkSmartPointer<vtkProgressBarWidget>::New();
-    interactor->SetInteractorOn(this->ProgressWidget);
+    this->Interactor->SetInteractorOn(this->ProgressWidget);
 
     vtkProgressBarRepresentation* progressRep =
       vtkProgressBarRepresentation::SafeDownCast(this->ProgressWidget->GetRepresentation());
@@ -48,13 +74,9 @@ bool animationManager::Initialize(
     progressRep->DrawBackgroundOff();
     progressRep->DragableOff();
     progressRep->SetShowBorderToOff();
-// Complete vtkProgressBarRepresentation needs
-// https://gitlab.kitware.com/vtk/vtk/-/merge_requests/7359
-#if VTK_VERSION_NUMBER >= VTK_VERSION_CHECK(9, 0, 20201027)
     progressRep->DrawFrameOff();
     progressRep->SetPadding(0.0, 0.0);
-#endif
-
+    progressRep->SetVisibility(this->Options.ui.animation_progress);
     this->ProgressWidget->On();
   }
   else
@@ -62,46 +84,33 @@ bool animationManager::Initialize(
     this->ProgressWidget = nullptr;
   }
 
-  int animationIndex = options->getAsInt("scene.animation.index");
-  double animationTime = options->getAsDouble("scene.animation.time");
-
   if (this->AvailAnimations <= 0)
   {
-    log::debug("No animation available in this file");
-    if (animationIndex > 0)
+    log::debug("No animation available");
+    if (this->Options.scene.animation.index > 0)
     {
       log::warn("An animation index has been specified but there are no animation available.");
     }
-    if (animationTime != 0)
-    {
-      log::warn("No animation available, cannot load a specific animation time");
-    }
-
-    this->HasAnimation = false;
-    return false;
+    return;
   }
   else
   {
-    log::debug("Animation(s) available in this file are:");
+    log::debug("Animation(s) available are:");
   }
   for (int i = 0; i < this->AvailAnimations; i++)
   {
     log::debug(i, ": ", this->Importer->GetAnimationName(i));
   }
-  log::debug("");
 
-  this->AnimationIndex = options->getAsInt("scene.animation.index");
+  this->AnimationIndex = this->Options.scene.animation.index;
   if (this->AnimationIndex > 0 && this->AnimationIndex >= this->AvailAnimations)
   {
     log::warn(
       "Specified animation index is greater than the highest possible animation index, enabling "
       "the first animation.");
-    this->Importer->EnableAnimation(0);
-  }
-  else
-  {
-    this->EnableOnlyCurrentAnimation();
+    this->AnimationIndex = 0;
   }
+  this->EnableOnlyCurrentAnimation();
 
   // Recover time ranges for all enabled animations
   this->TimeRange[0] = std::numeric_limits<double>::infinity();
@@ -114,17 +123,11 @@ bool animationManager::Initialize(
       int nbTimeSteps;
       vtkNew<vtkDoubleArray> timeSteps;
 
-// Complete GetTemporalInformation needs https://gitlab.kitware.com/vtk/vtk/-/merge_requests/7246
-#if VTK_VERSION_NUMBER >= VTK_VERSION_CHECK(9, 0, 20201016)
-      // Discard timesteps, F3D only cares about real elapsed time using time range
-      // Specifying the frame rate in the next call is not needed after VTK 9.2.20230603 :
+      // Discard timesteps, F3D only cares about elapsed time using time range and deltaTime
+      // Specifying a frame rate (60) in the next call is not needed after VTK 9.2.20230603 :
       // VTK_VERSION_CHECK(9, 2, 20230603)
-      double frameRate = this->Options->getAsDouble("scene.animation.frame-rate");
-      this->Importer->GetTemporalInformation(
-        animIndex, frameRate, nbTimeSteps, timeRange, timeSteps);
-#else
-      this->Importer->GetTemporalInformation(animIndex, nbTimeSteps, timeRange, timeSteps);
-#endif
+      this->Importer->GetTemporalInformation(animIndex, 60, nbTimeSteps, timeRange, timeSteps);
+
       // Accumulate time ranges
       this->TimeRange[0] = std::min(timeRange[0], this->TimeRange[0]);
       this->TimeRange[1] = std::max(timeRange[1], this->TimeRange[1]);
@@ -136,19 +139,19 @@ bool animationManager::Initialize(
     log::warn("Animation(s) time range delta is invalid: [", this->TimeRange[0], ", ",
       this->TimeRange[1], "]. Disabling animation.");
     this->HasAnimation = false;
-    return false;
+    return;
   }
   else
   {
     log::debug("Animation(s) time range is: [", this->TimeRange[0], ", ", this->TimeRange[1], "].");
   }
+  log::debug("");
 
-  bool autoplay = options->getAsBool("scene.animation.autoplay");
+  bool autoplay = this->Options.scene.animation.autoplay;
   if (autoplay)
   {
     this->StartAnimation();
   }
-  return true;
 }
 
 //----------------------------------------------------------------------------
@@ -167,7 +170,6 @@ void animationManager::StopAnimation()
   {
     this->ToggleAnimation();
   }
-  this->ProgressWidget = nullptr;
 }
 
 //----------------------------------------------------------------------------
@@ -177,10 +179,6 @@ void animationManager::ToggleAnimation()
   {
     this->Playing = !this->Playing;
 
-    if (this->CallBackId != 0)
-    {
-      this->Interactor->removeTimerCallBack(this->CallBackId);
-    }
     if (this->Playing)
     {
       // Initialize time if not already
@@ -192,13 +190,9 @@ void animationManager::ToggleAnimation()
 
       // Always reset previous tick when starting the animation
       this->PreviousTick = std::chrono::steady_clock::now();
-
-      double frameRate = this->Options->getAsDouble("scene.animation.frame-rate");
-      this->CallBackId =
-        this->Interactor->createTimerCallBack(1000.0 / frameRate, [this]() { this->Tick(); });
     }
 
-    if (this->Playing && this->Options->getAsInt("scene.camera.index") >= 0)
+    if (this->Playing && this->Options.scene.camera.index.has_value())
     {
       this->Interactor->disableCameraMovement();
     }
@@ -212,36 +206,38 @@ void animationManager::ToggleAnimation()
 //----------------------------------------------------------------------------
 void animationManager::Tick()
 {
-  assert(this->Interactor);
-
-  // Compute time since previous tick
-  std::chrono::steady_clock::time_point tick = std::chrono::steady_clock::now();
-  auto timeInMS =
-    std::chrono::duration_cast<std::chrono::milliseconds>(tick - this->PreviousTick).count();
-  this->PreviousTick = tick;
-
-  // Convert to a usable time in seconds
-  double elapsedTime = static_cast<double>(timeInMS) / 1000.0;
-  double animationSpeedFactor = this->Options->getAsDouble("scene.animation.speed-factor");
-
-  // elapsedTime can be negative
-  elapsedTime *= animationSpeedFactor;
-  this->CurrentTime += elapsedTime;
-
-  // Modulo computation, compute CurrentTime in the time range.
-  if (this->CurrentTime < this->TimeRange[0] || this->CurrentTime > this->TimeRange[1])
+  if (this->Playing)
   {
-    auto modulo = [](double val, double mod)
+    std::chrono::steady_clock::time_point currentTick = std::chrono::steady_clock::now();
+    if (this->SkipFrame &&
+      std::chrono::duration_cast<std::chrono::milliseconds>(currentTick - this->PreviousTick)
+          .count() > 1000 * this->DeltaTime)
     {
-      const double remainder = fmod(val, mod);
-      return remainder < 0 ? remainder + mod : remainder;
-    };
-    this->CurrentTime = this->TimeRange[0] +
-      modulo(this->CurrentTime - this->TimeRange[0], this->TimeRange[1] - this->TimeRange[0]);
-  }
+      log::debug("Unable to process frames fast enough, skipping a frame");
+      this->PreviousTick = currentTick;
+      return;
+    }
+    this->PreviousTick = currentTick;
 
-  this->LoadAtTime(this->CurrentTime);
-  this->Window->render();
+    this->CurrentTime += this->DeltaTime * this->Options.scene.animation.speed_factor;
+
+    // Modulo computation, compute CurrentTime in the time range.
+    if (this->CurrentTime < this->TimeRange[0] || this->CurrentTime > this->TimeRange[1])
+    {
+      auto modulo = [](double val, double mod)
+      {
+        const double remainder = fmod(val, mod);
+        return remainder < 0 ? remainder + mod : remainder;
+      };
+      this->CurrentTime = this->TimeRange[0] +
+        modulo(this->CurrentTime - this->TimeRange[0], this->TimeRange[1] - this->TimeRange[0]);
+    }
+
+    if (this->LoadAtTime(this->CurrentTime))
+    {
+      this->Window.render();
+    }
+  }
 }
 
 //----------------------------------------------------------------------------
@@ -250,19 +246,37 @@ bool animationManager::LoadAtTime(double timeValue)
   assert(this->Importer);
   if (!this->HasAnimation)
   {
+    log::warn("No animation available, cannot load a specific animation time");
     return false;
   }
-  if (timeValue < this->TimeRange[0] || timeValue > this->TimeRange[1])
+
+  /* clamp target time to available range */
+  if (timeValue < this->TimeRange[0])
   {
-    log::warn("Provided time value: ", timeValue, " is outside of animation time range: [",
-      this->TimeRange[0], ", ", this->TimeRange[1], "] .");
-    return false;
+    log::warn("Animation time ", timeValue, " is outside of range [", this->TimeRange[0], ", ",
+      this->TimeRange[1], "], using ", this->TimeRange[0], ".");
+    timeValue = this->TimeRange[0];
   }
+  else if (timeValue > this->TimeRange[1])
+  {
+    log::warn("Animation time ", timeValue, " is outside of range [", this->TimeRange[0], ", ",
+      this->TimeRange[1], "], using ", this->TimeRange[1], ".");
+    timeValue = this->TimeRange[1];
+  }
+
   this->CurrentTime = timeValue;
   this->CurrentTimeSet = true;
+#if VTK_VERSION_NUMBER >= VTK_VERSION_CHECK(9, 3, 20240707)
+  if (!this->Importer->UpdateAtTimeValue(this->CurrentTime))
+  {
+    log::error("Could not load time value: ", this->CurrentTime);
+    return false;
+  }
+#else
   this->Importer->UpdateTimeStep(this->CurrentTime);
+#endif
 
-  if (this->Interactor)
+  if (this->Interactor && this->ProgressWidget)
   {
     // Set progress bar
     vtkProgressBarRepresentation* progressRep =
@@ -292,6 +306,10 @@ void animationManager::CycleAnimation()
 
   this->EnableOnlyCurrentAnimation();
   this->LoadAtTime(this->TimeRange[0]);
+
+  vtkRenderWindow* renWin = this->Window.GetRenderWindow();
+  vtkF3DRenderer* ren = vtkF3DRenderer::SafeDownCast(renWin->GetRenderers()->GetFirstRenderer());
+  ren->SetCheatSheetConfigured(false);
 }
 
 // ---------------------------------------------------------------------------------
@@ -303,12 +321,13 @@ int animationManager::GetAnimationIndex()
 // ---------------------------------------------------------------------------------
 std::string animationManager::GetAnimationName()
 {
-  if (!this->Importer || this->AvailAnimations <= 0)
+  assert(this->Importer);
+  if (this->AvailAnimations <= 0)
   {
     return "No animation";
   }
 
-  if (this->AnimationIndex == -1)
+  if (this->AnimationIndex < 0)
   {
     return "All Animations";
   }
@@ -325,7 +344,7 @@ void animationManager::EnableOnlyCurrentAnimation()
   }
   for (int i = 0; i < this->AvailAnimations; i++)
   {
-    if (this->AnimationIndex == -1 || i == this->AnimationIndex)
+    if (this->AnimationIndex < 0 || i == this->AnimationIndex)
     {
       this->Importer->EnableAnimation(i);
     }
@@ -333,9 +352,8 @@ void animationManager::EnableOnlyCurrentAnimation()
 }
 
 //----------------------------------------------------------------------------
-void animationManager::GetTimeRange(double timeRange[2])
+std::pair<double, double> animationManager::GetTimeRange()
 {
-  timeRange[0] = this->TimeRange[0];
-  timeRange[1] = this->TimeRange[1];
+  return std::make_pair(this->TimeRange[0], this->TimeRange[1]);
 }
 }
