@@ -1,14 +1,16 @@
+#include <pybind11/functional.h>
 #include <pybind11/operators.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
+#include <pybind11/stl/filesystem.h>
 
 #include "camera.h"
 #include "engine.h"
 #include "image.h"
 #include "interactor.h"
-#include "loader.h"
 #include "log.h"
 #include "options.h"
+#include "scene.h"
 #include "types.h"
 #include "utils.h"
 #include "window.h"
@@ -117,7 +119,7 @@ PYBIND11_MODULE(pyf3d, module)
 
   image //
     .def(py::init<>())
-    .def(py::init<const std::string&>())
+    .def(py::init<const std::filesystem::path&>())
     .def(py::init<unsigned int, unsigned int, unsigned int, f3d::image::ChannelType>())
     .def(py::self == py::self)
     .def(py::self != py::self)
@@ -137,13 +139,13 @@ PYBIND11_MODULE(pyf3d, module)
     .def("to_terminal_text", [](const f3d::image& img) { return img.toTerminalText(); })
     .def("set_metadata", &f3d::image::setMetadata)
     .def("get_metadata",
-      [](const f3d::image& img, const std::string& key)
+      [](const f3d::image& img, std::string key)
       {
         try
         {
-          return img.getMetadata(key);
+          return img.getMetadata(std::move(key));
         }
-        catch (std::out_of_range e)
+        catch (const f3d::image::metadata_exception&)
         {
           throw py::key_error(key);
         }
@@ -156,39 +158,47 @@ PYBIND11_MODULE(pyf3d, module)
 
   options //
     .def(py::init<>())
-    .def("__setitem__", py::overload_cast<const std::string&, bool>(&f3d::options::set))
-    .def("__setitem__", py::overload_cast<const std::string&, int>(&f3d::options::set))
-    .def("__setitem__", py::overload_cast<const std::string&, double>(&f3d::options::set))
-    .def(
-      "__setitem__", py::overload_cast<const std::string&, const std::string&>(&f3d::options::set))
     .def("__setitem__",
-      py::overload_cast<const std::string&, const std::vector<int>&>(&f3d::options::set))
-    .def("__setitem__",
-      py::overload_cast<const std::string&, const std::vector<double>&>(&f3d::options::set))
-    .def("__getitem__",
-      [](f3d::options& opts, const std::string key)
+      [](f3d::options& opts, const std::string& key, const f3d::option_variant_t& value)
       {
-#define TRY(getter)                                                                                \
-  try                                                                                              \
-  {                                                                                                \
-    auto v = getter(key);                                                                          \
-    return py::cast(v);                                                                            \
-  }                                                                                                \
-  catch (const f3d::options::inexistent_exception&)                                                \
-  {                                                                                                \
-    throw pybind11::key_error(key);                                                                \
-  }                                                                                                \
-  catch (const f3d::options::incompatible_exception&)                                              \
-  {                                                                                                \
-  }
-        TRY(opts.getAsBoolRef)
-        TRY(opts.getAsIntRef)
-        TRY(opts.getAsDoubleRef)
-        TRY(opts.getAsStringRef)
-        TRY(opts.getAsDoubleVectorRef)
-        TRY(opts.getAsIntVectorRef)
-        throw pybind11::key_error(key);
-#undef TRY
+        try
+        {
+          opts.set(key, value);
+        }
+        catch (const f3d::options::inexistent_exception&)
+        {
+          throw py::key_error(key);
+        }
+        catch (const f3d::options::incompatible_exception&)
+        {
+          if (std::holds_alternative<std::string>(value))
+          {
+            try
+            {
+              opts.setAsString(key, std::get<std::string>(value));
+            }
+            catch (const f3d::options::parsing_exception&)
+            {
+              throw py::value_error(std::get<std::string>(value));
+            }
+          }
+          else
+          {
+            throw py::attribute_error(key);
+          }
+        }
+      })
+    .def("__getitem__",
+      [](f3d::options& opts, const std::string& key)
+      {
+        try
+        {
+          return opts.get(key);
+        }
+        catch (const f3d::options::inexistent_exception&)
+        {
+          throw py::key_error(key);
+        }
       })
     .def("__len__", [](f3d::options& opts) { return opts.getNames().size(); })
     .def(
@@ -209,9 +219,25 @@ PYBIND11_MODULE(pyf3d, module)
   py::class_<f3d::utils> utils(module, "Utils");
 
   utils //
-    .def_static("text_distance", &f3d::utils::textDistance);
+    .def_static("text_distance", &f3d::utils::textDistance)
+    .def_static("collapse_path", &f3d::utils::collapsePath);
 
   // f3d::interactor
+  py::class_<f3d::interaction_bind_t> interaction_bind(module, "InteractionBind");
+
+  py::enum_<f3d::interaction_bind_t::ModifierKeys>(interaction_bind, "ModifierKeys")
+    .value("ANY", f3d::interaction_bind_t::ModifierKeys::ANY)
+    .value("NONE", f3d::interaction_bind_t::ModifierKeys::NONE)
+    .value("CTRL", f3d::interaction_bind_t::ModifierKeys::CTRL)
+    .value("SHIFT", f3d::interaction_bind_t::ModifierKeys::SHIFT)
+    .value("CTRL_SHIFT", f3d::interaction_bind_t::ModifierKeys::CTRL_SHIFT)
+    .export_values();
+
+  interaction_bind.def(py::init<const f3d::interaction_bind_t::ModifierKeys&, const std::string&>())
+    .def_readwrite("mod", &f3d::interaction_bind_t::mod)
+    .def_readwrite("inter", &f3d::interaction_bind_t::inter)
+    .def("format", &f3d::interaction_bind_t::format);
+
   py::class_<f3d::interactor, std::unique_ptr<f3d::interactor, py::nodelete>> interactor(
     module, "Interactor");
   interactor //
@@ -226,9 +252,32 @@ PYBIND11_MODULE(pyf3d, module)
       "Disable the camera interaction")
     .def("play_interaction", &f3d::interactor::playInteraction, "Play an interaction file")
     .def("record_interaction", &f3d::interactor::recordInteraction, "Record an interaction file")
-    .def("start", &f3d::interactor::start, "Start the interactor")
-    .def("stop", &f3d::interactor::start, "Stop the interactor")
-    .def_static("get_default_interactions_info", &f3d::interactor::getDefaultInteractionsInfo);
+    .def("start", &f3d::interactor::start, "Start the interactor and the event loop",
+      py::arg("delta_time") = 1.0 / 30, py::arg("user_callback") = nullptr)
+    .def("stop", &f3d::interactor::stop, "Stop the interactor and the event loop")
+    .def(
+      "request_render", &f3d::interactor::requestRender, "Request a render on the next event loop")
+    .def("init_commands", &f3d::interactor::initCommands,
+      "Remove all commands and add all default command callbacks")
+    .def("add_command", &f3d::interactor::addCommand, "Add a command")
+    .def("remove_command", &f3d::interactor::removeCommand, "Remove a command")
+    .def("get_command_actions", &f3d::interactor::getCommandActions, "Get all command actions")
+    .def("trigger_command", &f3d::interactor::triggerCommand, "Trigger a command")
+    .def("init_bindings", &f3d::interactor::initBindings,
+      "Remove all bindings and add default bindings")
+    .def("add_binding",
+      py::overload_cast<const f3d::interaction_bind_t&, std::string, std::string,
+        std::function<std::pair<std::string, std::string>()>>(&f3d::interactor::addBinding),
+      "Add a binding command")
+    .def("add_binding",
+      py::overload_cast<const f3d::interaction_bind_t&, std::vector<std::string>, std::string,
+        std::function<std::pair<std::string, std::string>()>>(&f3d::interactor::addBinding),
+      "Add binding commands")
+    .def("remove_binding", &f3d::interactor::removeBinding, "Remove interaction commands")
+    .def("get_bind_groups", &f3d::interactor::getBindGroups)
+    .def("get_binds_for_group", &f3d::interactor::getBindsForGroup)
+    .def("get_binds", &f3d::interactor::getBinds)
+    .def("get_binding_documentation", &f3d::interactor::getBindingDocumentation);
 
   // f3d::mesh_t
   py::class_<f3d::mesh_t>(module, "Mesh")
@@ -245,16 +294,32 @@ PYBIND11_MODULE(pyf3d, module)
     .def_readwrite("face_sides", &f3d::mesh_t::face_sides)
     .def_readwrite("face_indices", &f3d::mesh_t::face_indices);
 
-  // f3d::loader
-  py::class_<f3d::loader, std::unique_ptr<f3d::loader, py::nodelete>> loader(module, "Loader");
-  loader //
-    .def("has_geometry_reader", &f3d::loader::hasGeometryReader)
-    .def("load_geometry", py::overload_cast<const std::string&, bool>(&f3d::loader::loadGeometry),
-      "load geometry to a default scene", py::arg("file_path"), py::arg("reset") = false)
-    .def("has_scene_reader", &f3d::loader::hasSceneReader)
-    .def("load_scene", &f3d::loader::loadScene, "Load a specific full scene file")
-    .def("load_geometry", py::overload_cast<const f3d::mesh_t&, bool>(&f3d::loader::loadGeometry),
-      "Load a surfacic mesh from memory", py::arg("mesh"), py::arg("reset") = false);
+  // f3d::scene
+  py::class_<f3d::scene, std::unique_ptr<f3d::scene, py::nodelete>> scene(module, "Scene");
+  scene //
+    .def("supports", &f3d::scene::supports)
+    .def("clear", &f3d::scene::clear)
+    .def("add", py::overload_cast<const std::filesystem::path&>(&f3d::scene::add),
+      "Add a file the scene", py::arg("file_path"))
+    .def("add", py::overload_cast<const std::vector<std::filesystem::path>&>(&f3d::scene::add),
+      "Add multiple filepaths to the scene", py::arg("file_path_vector"))
+    .def("add", py::overload_cast<const std::vector<std::string>&>(&f3d::scene::add),
+      "Add multiple filenames to the scene", py::arg("file_name_vector"))
+    .def("add", py::overload_cast<const f3d::mesh_t&>(&f3d::scene::add),
+      "Add a surfacic mesh from memory into the scene", py::arg("mesh"))
+    .def("load_animation_time", &f3d::scene::loadAnimationTime)
+    .def("animation_time_range", &f3d::scene::animationTimeRange);
+
+  // f3d::camera_state_t
+  py::class_<f3d::camera_state_t>(module, "CameraState")
+    .def(py::init<>())
+    .def(py::init<const f3d::point3_t&, const f3d::point3_t&, const f3d::vector3_t&,
+           const f3d::angle_deg_t&>(),
+      py::arg("position"), py::arg("focal_point"), py::arg("view_up"), py::arg("view_angle"))
+    .def_readwrite("position", &f3d::camera_state_t::position)
+    .def_readwrite("focal_point", &f3d::camera_state_t::focalPoint)
+    .def_readwrite("view_up", &f3d::camera_state_t::viewUp)
+    .def_readwrite("view_angle", &f3d::camera_state_t::viewAngle);
 
   // f3d::camera
   py::class_<f3d::camera, std::unique_ptr<f3d::camera, py::nodelete>> camera(module, "Camera");
@@ -282,27 +347,23 @@ PYBIND11_MODULE(pyf3d, module)
     .def("reset_to_default", &f3d::camera::resetToDefault)
     .def("reset_to_bounds", &f3d::camera::resetToBounds, py::arg("zoom_factor") = 0.9);
 
-  py::class_<f3d::camera_state_t>(module, "CameraState")
-    .def(py::init<>())
-    .def(py::init<const f3d::point3_t&, const f3d::point3_t&, const f3d::vector3_t&,
-      const f3d::angle_deg_t&>())
-    .def_readwrite("pos", &f3d::camera_state_t::pos)
-    .def_readwrite("foc", &f3d::camera_state_t::foc)
-    .def_readwrite("up", &f3d::camera_state_t::up)
-    .def_readwrite("angle", &f3d::camera_state_t::angle);
-
   // f3d::window
   py::class_<f3d::window, std::unique_ptr<f3d::window, py::nodelete>> window(module, "Window");
 
   py::enum_<f3d::window::Type>(window, "Type")
     .value("NONE", f3d::window::Type::NONE)
-    .value("NATIVE", f3d::window::Type::NATIVE)
-    .value("NATIVE_OFFSCREEN", f3d::window::Type::NATIVE_OFFSCREEN)
     .value("EXTERNAL", f3d::window::Type::EXTERNAL)
+    .value("GLX", f3d::window::Type::GLX)
+    .value("WGL", f3d::window::Type::WGL)
+    .value("COCOA", f3d::window::Type::COCOA)
+    .value("EGL", f3d::window::Type::EGL)
+    .value("OSMESA", f3d::window::Type::OSMESA)
+    .value("UNKNOWN", f3d::window::Type::UNKNOWN)
     .export_values();
 
   window //
     .def_property_readonly("type", &f3d::window::getType)
+    .def_property_readonly("offscreen", &f3d::window::isOffscreen)
     .def_property_readonly("camera", &f3d::window::getCamera, py::return_value_policy::reference)
     .def_property(
       "size",
@@ -324,35 +385,72 @@ PYBIND11_MODULE(pyf3d, module)
     .def("get_display_from_world", &f3d::window::getDisplayFromWorld,
       "Get display coordinate point from world coordinate");
 
+  // libInformation
+  py::class_<f3d::engine::libInformation>(module, "LibInformation")
+    .def_readonly("version", &f3d::engine::libInformation::Version)
+    .def_readonly("version_full", &f3d::engine::libInformation::VersionFull)
+    .def_readonly("build_date", &f3d::engine::libInformation::BuildDate)
+    .def_readonly("build_system", &f3d::engine::libInformation::BuildSystem)
+    .def_readonly("compiler", &f3d::engine::libInformation::Compiler)
+    .def_readonly("modules", &f3d::engine::libInformation::Modules)
+    .def_readonly("vtk_version", &f3d::engine::libInformation::VTKVersion)
+    .def_readonly("copyrights", &f3d::engine::libInformation::Copyrights)
+    .def_readonly("license", &f3d::engine::libInformation::License);
+
+  // readerInformation
+  py::class_<f3d::engine::readerInformation>(module, "ReaderInformation")
+    .def_readonly("name", &f3d::engine::readerInformation::Name)
+    .def_readonly("description", &f3d::engine::readerInformation::Description)
+    .def_readonly("extensions", &f3d::engine::readerInformation::Extensions)
+    .def_readonly("mime_types", &f3d::engine::readerInformation::MimeTypes)
+    .def_readonly("plugin_name", &f3d::engine::readerInformation::PluginName)
+    .def_readonly("has_scene_reader", &f3d::engine::readerInformation::HasSceneReader)
+    .def_readonly("has_geometry_reader", &f3d::engine::readerInformation::HasGeometryReader);
+
   // f3d::engine
   py::class_<f3d::engine> engine(module, "Engine");
 
   engine //
-    .def(py::init<f3d::window::Type>(), py::arg("window_type") = f3d::window::Type::NATIVE)
+    .def_static("create", &f3d::engine::create, "Create an engine with a automatic window",
+      py::arg("offscreen") = false)
+    .def_static("create_none", &f3d::engine::createNone, "Create an engine with no window")
+    .def_static(
+      "create_glx", &f3d::engine::createGLX, "Create an engine with an GLX window (Linux only)")
+    .def_static(
+      "create_wgl", &f3d::engine::createWGL, "Create an engine with an WGL window (Windows only)")
+    .def_static("create_egl", &f3d::engine::createEGL,
+      "Create an engine with an EGL window (Windows/Linux only)")
+    .def_static("create_osmesa", &f3d::engine::createOSMesa,
+      "Create an engine with an OSMesa window (Windows/Linux only)")
+    .def_static("create_external_glx", &f3d::engine::createExternalGLX,
+      "Create an engine with an existing GLX context (Linux only)")
+    .def_static("create_external_wgl", &f3d::engine::createExternalWGL,
+      "Create an engine with an existing WGL context (Windows only)")
+    .def_static("create_external_cocoa", &f3d::engine::createExternalCOCOA,
+      "Create an engine with an existing COCOA context (macOS only)")
+    .def_static("create_external_egl", &f3d::engine::createExternalEGL,
+      "Create an engine with an existing EGL context (Windows/Linux only)")
+    .def_static("create_external_osmesa", &f3d::engine::createExternalOSMesa,
+      "Create an engine with an existing OSMesa context (Windows/Linux only)")
     .def("set_cache_path", &f3d::engine::setCachePath, "Set the cache path directory")
     .def_property("options", &f3d::engine::getOptions,
       py::overload_cast<const f3d::options&>(&f3d::engine::setOptions),
       py::return_value_policy::reference)
     .def_property_readonly("window", &f3d::engine::getWindow, py::return_value_policy::reference)
-    .def_property_readonly("loader", &f3d::engine::getLoader, py::return_value_policy::reference)
+    .def_property_readonly("scene", &f3d::engine::getScene, py::return_value_policy::reference)
     .def_property_readonly(
       "interactor", &f3d::engine::getInteractor, py::return_value_policy::reference)
     .def_static("load_plugin", &f3d::engine::loadPlugin, "Load a plugin")
     .def_static(
       "autoload_plugins", &f3d::engine::autoloadPlugins, "Automatically load internal plugins")
-    .def_static("get_plugins_list", &f3d::engine::getPluginsList);
+    .def_static("get_plugins_list", &f3d::engine::getPluginsList)
+    .def_static("get_lib_info", &f3d::engine::getLibInfo, py::return_value_policy::reference)
+    .def_static("get_readers_info", &f3d::engine::getReadersInfo)
+    .def_static("get_rendering_backend_list", &f3d::engine::getRenderingBackendList);
 
   // f3d::log
   py::class_<f3d::log> log(module, "Log");
 
-  log //
-    .def_static("set_verbose_level", &f3d::log::setVerboseLevel, py::arg("level"),
-      py::arg("force_std_err") = false)
-    .def_static("set_use_coloring", &f3d::log::setUseColoring)
-    .def_static("print",
-      [](f3d::log::VerboseLevel& level, const std::string& message)
-      { f3d::log::print(level, message); });
-
   py::enum_<f3d::log::VerboseLevel>(log, "VerboseLevel")
     .value("DEBUG", f3d::log::VerboseLevel::DEBUG)
     .value("INFO", f3d::log::VerboseLevel::INFO)
@@ -361,129 +459,11 @@ PYBIND11_MODULE(pyf3d, module)
     .value("QUIET", f3d::log::VerboseLevel::QUIET)
     .export_values();
 
-// deprecated functions, will be removed in the next major release, F3D v3.0.0
-#ifndef F3D_NO_DEPRECATED
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-  image //
-    .def("setResolution", &f3d::image::setResolution,
-      "DEPRECATED: use the appropriate constructor instead.")
-    .def("setChannelCount", &f3d::image::setResolution,
-      "DEPRECATED: use the appropriate constructor instead.")
-    .def(
-      "setData", [=](f3d::image& img, const py::bytes& data) { setImageBytes(img, data); },
-      "DEPRECATED: use setContent instead")
-    .def(
-      "getData", [=](const f3d::image& img) { return getImageBytes(img); },
-      "DEPRECATED: use getContent instead.")
-    .def("getWidth", &f3d::image::getWidth, "DEPRECATED")
-    .def("getHeight", &f3d::image::getHeight, "DEPRECATED")
-    .def("getChannelCount", &f3d::image::getChannelCount, "DEPRECATED")
-    .def("getChannelType", &f3d::image::getChannelType, "DEPRECATED")
-    .def("getChannelTypeSize", &f3d::image::getChannelTypeSize, "DEPRECATED")
-    .def("setContent", setImageBytes, "DEPRECATED")
-    .def("getContent", getImageBytes, "DEPRECATED");
-
-  options //
-    .def("set", py::overload_cast<const std::string&, bool>(&f3d::options::set),
-      "Set a boolean option", "DEPRECATED")
-    .def("set", py::overload_cast<const std::string&, int>(&f3d::options::set),
-      "Set an integer option", "DEPRECATED")
-    .def("set", py::overload_cast<const std::string&, double>(&f3d::options::set),
-      "Set a floating point option", "DEPRECATED")
-    .def("set", py::overload_cast<const std::string&, const std::string&>(&f3d::options::set),
-      "Set a string option", "DEPRECATED")
-    .def("set", py::overload_cast<const std::string&, const std::vector<int>&>(&f3d::options::set),
-      "Set an array of integers option", "DEPRECATED")
-    .def("set",
-      py::overload_cast<const std::string&, const std::vector<double>&>(&f3d::options::set),
-      "Set an array of floating points option", "DEPRECATED")
-    .def("getAsBool", &f3d::options::getAsBool, "DEPRECATED")
-    .def("getAsInt", &f3d::options::getAsInt, "DEPRECATED")
-    .def("getAsDouble", &f3d::options::getAsDouble, "DEPRECATED")
-    .def("getAsString", &f3d::options::getAsString, "DEPRECATED")
-    .def("getAsIntVector", &f3d::options::getAsIntVector, "DEPRECATED")
-    .def("getAsDoubleVector", &f3d::options::getAsDoubleVector, "DEPRECATED")
-    .def("isSame", &f3d::options::isSame, "DEPRECATED")
-    .def("getNames", &f3d::options::getNames, "DEPRECATED")
-    .def("getClosestOption", &f3d::options::getClosestOption, "DEPRECATED");
-
-  utils //
-    .def_static("textDistance", &f3d::utils::textDistance, "DEPRECATED");
-
-  interactor //
-    .def("toggleAnimation", &f3d::interactor::toggleAnimation, "Toggle the animation", "DEPRECATED")
-    .def("startAnimation", &f3d::interactor::startAnimation, "Start the animation", "DEPRECATED")
-    .def("stopAnimation", &f3d::interactor::stopAnimation, "Stop the animation", "DEPRECATED")
-    .def("isPlayingAnimation", &f3d::interactor::isPlayingAnimation,
-      "Returns True if the animation is currently started", "DEPRECATED")
-    .def("enableCameraMovement", &f3d::interactor::enableCameraMovement,
-      "Enable the camera interaction", "DEPRECATED")
-    .def("disableCameraMovement", &f3d::interactor::disableCameraMovement,
-      "Disable the camera interaction", "DEPRECATED")
-    .def("playInteraction", &f3d::interactor::playInteraction, "Play an interaction file",
-      "DEPRECATED")
-    .def("recordInteraction", &f3d::interactor::recordInteraction, "Record an interaction file",
-      "DEPRECATED")
-    .def_static(
-      "getDefaultInteractionsInfo", &f3d::interactor::getDefaultInteractionsInfo, "DEPRECATED");
-
-  loader //
-    .def("hasGeometryReader", &f3d::loader::hasGeometryReader, "DEPRECATED")
-    .def("loadGeometry", py::overload_cast<const std::string&, bool>(&f3d::loader::loadGeometry),
-      "load geometry to a default scene", "DEPRECATED")
-    .def("hasSceneReader", &f3d::loader::hasSceneReader, "DEPRECATED")
-    .def("loadScene", &f3d::loader::loadScene, "Load a specific full scene file", "DEPRECATED");
-
-  camera //
-    .def("setPosition", &f3d::camera::setPosition, "DEPRECATED: use position property setter")
-    .def("getPosition", py::overload_cast<>(&f3d::camera::getPosition),
-      "DEPRECATED: use position property getter")
-    .def("setFocalPoint", &f3d::camera::setFocalPoint, "DEPRECATED")
-    .def("getFocalPoint", py::overload_cast<>(&f3d::camera::getFocalPoint), "DEPRECATED")
-    .def("setViewUp", &f3d::camera::setViewUp, "DEPRECATED")
-    .def("getViewUp", py::overload_cast<>(&f3d::camera::getViewUp), "DEPRECATED")
-    .def("setViewAngle", &f3d::camera::setViewAngle, "DEPRECATED")
-    .def("getViewAngle", py::overload_cast<>(&f3d::camera::getViewAngle), "DEPRECATED")
-    .def("getViewAngle", py::overload_cast<f3d::angle_deg_t&>(&f3d::camera::getViewAngle),
-      "DEPRECATED")
-    .def("setState", &f3d::camera::setState, "DEPRECATED")
-    .def("getState", py::overload_cast<>(&f3d::camera::getState), "DEPRECATED")
-    .def("getState", py::overload_cast<f3d::camera_state_t&>(&f3d::camera::getState), "DEPRECATED")
-    .def("setCurrentAsDefault", &f3d::camera::setCurrentAsDefault, "DEPRECATED")
-    .def("resetToDefault", &f3d::camera::resetToDefault, "DEPRECATED")
-    .def("resetToBounds", &f3d::camera::resetToBounds, py::arg("zoomFactor") = 0.9, "DEPRECATED");
-
-  window //
-    .def("getType", &f3d::window::getType, "DEPRECATED")
-    .def("getCamera", &f3d::window::getCamera, py::return_value_policy::reference, "DEPRECATED")
-    .def("renderToImage", &f3d::window::renderToImage, "Render the window to an image",
-      py::arg("noBackground") = false, "DEPRECATED")
-    .def("setSize", &f3d::window::setSize, "Set the window size", "DEPRECATED")
-    .def("getWidth", &f3d::window::getWidth, "Get the window width", "DEPRECATED")
-    .def("getHeight", &f3d::window::getHeight, "Get the window height", "DEPRECATED")
-    .def("setPosition", &f3d::window::setPosition, "DEPRECATED")
-    .def("setIcon", &f3d::window::setIcon,
-      "Set the icon of the window using a memory buffer representing a PNG file", "DEPRECATED")
-    .def("setWindowName", &f3d::window::setWindowName, "Set the window name", "DEPRECATED")
-    .def("getWorldFromDisplay", &f3d::window::getWorldFromDisplay,
-      "Get world coordinate point from display coordinate", "DEPRECATED")
-    .def("getDisplayFromWorld", &f3d::window::getDisplayFromWorld,
-      "Get display coordinate point from world coordinate", "DEPRECATED");
-
-  engine //
-    .def("setCachePath", &f3d::engine::setCachePath, "Set the cache path directory", "DEPRECATED")
-    .def(
-      "setOptions", py::overload_cast<const f3d::options&>(&f3d::engine::setOptions), "DEPRECATED")
-    .def("getOptions", &f3d::engine::getOptions, py::return_value_policy::reference, "DEPRECATED")
-    .def("getWindow", &f3d::engine::getWindow, py::return_value_policy::reference, "DEPRECATED")
-    .def("getLoader", &f3d::engine::getLoader, py::return_value_policy::reference, "DEPRECATED")
-    .def("getInteractor", &f3d::engine::getInteractor, py::return_value_policy::reference,
-      "DEPRECATED")
-    .def_static("loadPlugin", &f3d::engine::loadPlugin, "Load a plugin", "DEPRECATED")
-    .def_static("autoloadPlugins", &f3d::engine::autoloadPlugins,
-      "Automatically load internal plugins", "DEPRECATED")
-    .def_static("getPluginsList", &f3d::engine::getPluginsList, "DEPRECATED");
-#pragma GCC diagnostic pop
-#endif
+  log //
+    .def_static("set_verbose_level", &f3d::log::setVerboseLevel, py::arg("level"),
+      py::arg("force_std_err") = false)
+    .def_static("set_use_coloring", &f3d::log::setUseColoring)
+    .def_static("print",
+      [](f3d::log::VerboseLevel& level, const std::string& message)
+      { f3d::log::print(level, message); });
 }
